;+
; NAME:
;   SCATTER3D
;
; PURPOSE:
;      Show a scatterplot of two arrays with color coding to
;      indicate density of points.
;
; AUTHOR:
;
;   Thomas Berger
;   Lockheed Martin Solar and Astrophysics Laboratory
;   berger@lmsal.com
;
; CATEGORY:
;       Data analysis, graphics.
;
; CALLING SEQUENCE:
;       SCATTER3D,array1,array2
;
; INPUTS:
;       ARRAY1:   data array of any dimension > 0
;       ARRAY2:   data array of the same dimension as ARRAY1
;
; OUTPUTS:
;       BINS:     The binsize actually used in HIST2D (determined from
;                 data ranges and NBIN parameter.
;
;       R:        The HIST2D "image" used to display the scatterplot
;
;       CENTER:   The center of the principle moment ellipse. Only if
;                 keyword ELLIPSE is set.
;
;      SEMI_AIXS: FLTARR[2] containing the semi-axes of the principle
;                 moment ellipse in data units. Only if keyword ELLIPSE
;                 is set.
;
;       ROTANG:   Rotation angle of the principle moment ellipse in
;                 degrees counterclockwise from x-axis. Only if
;                 keyword ELLIPSE is set.
;
; KEYWORD PARAMETERS:
;
;       NBIN:     The number of bins for HIST2D - effectively the
;                 number of "pixels" in the scatterplot image.
;                 Default is 100.
;
;       ELLIPSE:  If set, routine calculates and plots the principle =
moment
;                 ellipse fit to the scatterplot "image".
;
;       XTITLE:   Passed directly to PLOT.PRO for axis labelling.
;       YTITLE
;
;       PLOT45:   If set, plots the 45-degree, slope=3D1, line for =
comparison
;                 to the scatterplot data.
;
; COMMON BLOCKS:
;       None.
;
; SIDE EFFECTS:
;       720x540 pixel direct graphics window is opened in upper right of =
display.
;
; RESTRICTIONS:
;       Requires HIST2D.PRO (built-in IDL function) and
;       COLORBAR.PRO (Fanning Software IDL procedure, also found in =
SolarSoft).
;
; EXAMPLE:
;       To make a scatterplot of two images IMAGE1 and IMAGE2 with
;       the priniciple moment ellipse and a 45-degree line
;
;       SCATTER3D, image1, image2, bins, r, center, semi, rotang,$
;           /ELLIPSE,/PLOT45,XTITLE=3D'Image 1', YTITLE=3D'Image 2'
;
; MODIFICATION HISTORY:
;       Written by: Thomas Berger, LMSAL, 10/2001.
;-
;

PRO scatter3d, dat1, dat2, bins, r, center, semi_axis, rotang, $
               NBIN=3Dnbin, ELLIPSE=3Dellipse, $
               XTITLE=3Dxtitle, YTITLE=3Dytitle, $
               PLOT45=3Dplot45

ON_ERROR,2

s1 =3D SIZE(dat1)
s2 =3D SIZE(dat1)
if s2(0) ne s1(0) OR s2(1) ne s1(1) OR s2(2) ne s1(2) then begin
    MESSAGE,'Dimensions of datasets unequal - returning.'
    RETURN
end

if KEYWORD_SET(xtitle) then xt =3D xtitle else xt =3D 'DATA1'
if KEYWORD_SET(ytitle) then yt =3D ytitle else yt =3D 'DATA2'

mx1 =3D FLOAT(MAX(dat1))
mn1 =3D FLOAT(MIN(dat1))
if KEYWORD_SET(nbin) then ng =3D nbin else ng=3D100.
rng1 =3D mx1-mn1
b1 =3D ROUND(ng*rng1)/FLOAT(ng)/ng

mx2 =3D FLOAT(MAX(dat2))
mn2 =3D FLOAT(MIN(dat2))
rng2 =3D mx2-mn2
b2 =3D ROUND(ng*rng2)/FLOAT(ng)/ng

mins =3D [mn1,mn2]

r =3D =
HIST_2D(dat1,dat2,BIN1=3Db1,BIN2=3Db2,MIN1=3Dmn1,MAX1=3Dmx1,MIN2=3Dmn2,MA=
X2=3Dmx2)
szr =3D SIZE(r)
;actual binsize of x and y pixels in r:
bins =3D [rng1/szr[1],rng2/szr[2]]

;Make the FIXED ASPECT RATIO plot
WINDOW,/free,xsize=3D720,ysize=3D540
LOADCT,2
ar =3D FLOAT(680)/640
x0 =3D 0.15
y0 =3D 0.10
x1 =3D 0.76
y1 =3D y0 + (x1-x0)*ar
!p.position=3D[x0,y0,x1,y1]
y1 =3D ((x1-x0)*!d.x_vsize)/((y1-y0)*!d.y_vsize)*y1
!p.position=3D[x0,y0,x1,y1]
!x.margin=3D[0,4]
!y.margin=3D[4,4]

PLOT,[mn1,mx1],[mn2,mx2],/nodata,$
  xrange=3Dxrang,yrange=3Dyrang,/xsty,/ysty
px =3D !X.WINDOW*!D.X_VSIZE
py =3D !Y.WINDOW*!D.Y_VSIZE
sx =3D px[1]-px[0]+1
sy =3D py[1]-py[0]+1
TVSCL,CONGRID(ALOG(r+1),sx,sy),px[0],py[0]
PLOT,[mn1,mx1],[mn2,mx2],/noerase,$
  /nodata,$
  xrange=3Dxrang,yrange=3Dyrang,/xsty,/ysty,$
  title=3D'Log(1+N) Histogram Scatterplot',xtitle=3Dxt,ytitle=3Dyt
PLOTS,[mn1,mx1],[0,0]
PLOTS,[0,0],[mn2,mx2]
;Plot 45-degree line for reference:
if KEYWORD_SET(plot45) then PLOTS,[mn1,mx1],[mn1,mx1]

;Add colorbar.
colorbar,position=3D[.90,.1,.94,y1],$
  range=3D[Min(r),Max(r)],/vertical,$
  format=3D'(I6)',title=3D'N Values',charsize=3D0.7

;Ellipse fit
if KEYWORD_SET(ellipse) then begin
    center =3D 0
    semi_axis =3D 0
    rotang =3D 0
    ellipse,r,center,semi_axis,rotang
    center =3D center*bins-ABS(mins)
    PLOTS,center[0],center[1],$
      psym=3D4,symsize=3D1,thick=3D3,color=3D0

    a =3D semi_axis[0]
    b =3D semi_axis[1]
    xp1 =3D FINDGEN(4*a)/2 - a
    xp1 =3D [xp1,-xp1[0]]
    yp =3D b*SQRT(1.0 - (xp1/a)^2.)
    yn =3D -yp

    cosp =3D COS(!DTOR*rotang)
    sinp =3D SIN(!DTOR*rotang)
    rotmat =3D [[sinp,cosp],[sinp,-cosp]]
    xypr =3D rotmat#TRANSPOSE([[xp1],[yp]])
    xynr =3D rotmat#TRANSPOSE([[xp1],[yn]])

;Draw the ellipse in the plot box only. Should use vectors but got =
lazy...
    xpr =3D REFORM(xypr[0,*])*bins[0]+center[0]
    ypr =3D REFORM(xypr[1,*])*bins[1]+center[1]
    goodx =3D WHERE(xpr ge mn1 and xpr le mx1)
    xpr =3D xpr[goodx]
    ypr =3D ypr[goodx]
    goody =3D WHERE(ypr ge mn2 and ypr le mx2)
    xpr =3D xpr[goody]
    ypr =3D ypr[goody]
    PLOTS,xpr,ypr, col=3D0, thick=3D2, lines=3D0

    goodx =3D WHERE(xynr[0,*] gt mn1 and xynr[0,*] lt mx1)
    xnr =3D REFORM(xynr[0,*])*bins[0]+center[0]
    ynr =3D REFORM(xynr[1,*])*bins[1]+center[1]
    goodx =3D WHERE(xnr ge mn1 and xnr le mx1)
    xnr =3D xnr[goodx]
    ynr =3D ynr[goodx]
    goody =3D WHERE(ynr ge mn2 and ynr le mx2)
    xnr =3D xnr[goody]
    ynr =3D ynr[goody]
    PLOTS,xnr,ynr, col=3D0, thick=3D2, lines=3D0

    semi_axis =3D semi_axis*bins
end

RETURN
END

PRO ellipse,eabt,center,semi_axes,rotation
;
;	eabt	  - 2-dimensional object array
;	center	  - center of computed ellipse (pixels) (centroid of eabt)
;	semi_axes - semi-axes of computed ellipse (pixels)
;	rotation  - rotation of computed ellipse wrt input array (degrees)
;
;Written by Bill Rosenberg 3/95.

sz=3Dsize(eabt)
if sz(0) ne 2 then begin
	print,' Dimension of input array, ',sz(0),' not equal to 2'
	retall
	endif
xd=3Dsz(1) & yd=3Dsz(2)
xr=3Dfindgen(xd,yd) mod xd
yr=3Dfindgen(yd,xd) mod yd
yr =3D transpose(yr)
;print,xr,format=3D'(<xd>i3)'
;print,yr,format=3D'(<yd>i3)'
s=3Dtotal(eabt)
xbar=3Dtotal(xr*eabt)/s
;print,xbar
ybar=3Dtotal(yr*eabt)/s
;print,ybar
sxx=3Dtotal(eabt*(xr-xbar)^2)/s
;print,sxx
syy=3Dtotal(eabt*(yr-ybar)^2)/s
;print,syy
sxy=3Dtotal(eabt*(xr-xbar)*(yr-ybar))/s
;print,sxy
;Covariance matrix:
kk=3D[[sxx,sxy],[sxy,syy]]
diagonal,kk,dd,t1
;print,2.*sqrt(dd > 0)
;print,t1
rotation=3Dt1
semi_axes=3D2.*sqrt([dd(0,0),dd(1,1)] > 0)
center=3D[xbar,ybar]
;print,' Ellipse center:    ',center
;print,' Ellipse semi-axes: ',semi_axes
;print,' Ellipse rotation:  ',rotation,' degrees'

RETURN
END

PRO diagonal,kk,dd,t1

;Matrix diagonalization routine. Used by ELLIPSE.PRO
; Rosenberg 3/95.
;print,kk
;      50.0000      22.0000
;      22.0000      10.0000
a=3Dkk(0,0) & b=3Dkk(0,1) & d=3Dkk(1,1)
;print,a*d-b^2
;      16.0000
l1=3D((a+d)+sqrt((a-d)^2+4.*b^2))/2.
;print,l1
;      59.7321
l2=3D((a+d)-sqrt((a-d)^2+4.*b^2))/2.
;print,l2
;     0.267862
t1=3Drtand(-b/(a-l1))
;print,t1
;      66.1368
t2=3Drtand(-b/(a-l2))
;print,t2
;     -23.8632
x1=3D[sind(t1),cosd(t1)]
x2=3D[sind(t2),cosd(t2)]
pp=3D[[x1],[x2]]
;print,pp
;     0.914514     0.404554
;    -0.404554     0.914514
ppi=3Dtranspose(pp)
dd=3Dppi#kk#pp
;print,dd
;      59.7321 2.942979e-06
; 3.814697e-06     0.267863

RETURN
END

